class ReportsController < InheritedResources::Base
  belongs_to :node, :optional => true

  # The master uploads reports using the upload method and does not have a
  # CSRF token, so if the method is upload we must skip verifying the token.
  skip_before_filter :verify_authenticity_token, :only => [:upload]

  before_filter :raise_if_enable_read_only_mode, :only => [:new, :edit, :update, :destroy]
  before_filter :handle_raw_post, :only => [:create, :upload]

  def index
    index! do |format|
      format.html do
        if params[:kind] == "inspect"
          @reports = paginate_scope Report.inspections
          @tab = false
        elsif Node.possible_statuses.include?(params[:status])
          @reports = paginate_scope Report.send(params[:status])
          @tab = params[:status]
        else
          @reports = paginate_scope Report.applies
          @tab = 'all'
        end
      end
    end
  end

  def by_node_and_time

    begin
      time = DateTime.parse(params[:report_time]).to_s(:db)
    rescue
      render :text => "Invalid report time", :status => :bad_request
      return
    end

    report = Report.where('host = ? AND date_trunc(\'second\', time) = ?', params[:certname], time).first!

    redirect_to report_path(report.id)
  end

  def create
    if SETTINGS.disable_legacy_report_upload_url
      render :text => "Access Denied, this url has been disabled, try /reports/upload", :status => 403
    else
      upload
    end
  end

  def upload
    begin
      Report.delay.create_from_yaml(params[:report][:report])
      render :text => "Report queued for import"
    rescue => e
      error_text = "ERROR! ReportsController#upload failed:"
      Rails.logger.debug error_text
      Rails.logger.debug e.message
      render :text => "#{error_text} #{e.message}", :status => 406
    end
  end

  private

  def collection
    get_collection_ivar || set_collection_ivar(
      request.format == :html ?
        paginate_scope(end_of_association_chain) :
        end_of_association_chain
    )
  end

  def handle_raw_post
    report = params[:report]
    params[:report] = {}
    case report
    when String
      params[:report][:report] = report
    when nil
      params[:report][:report] = request.raw_post
    when Hash
      params[:report] = report
    end
  end

end
